==> ricky_icon_generator-0.0.2.tar.gz: lib/commands/app_icon/android/android_app_icon_controller.dart <==
import 'dart:io';
import 'dart:typed_data';

import 'package:image/image.dart';
import 'package:meta/meta.dart';
import 'package:ricky_icon_generator/core/base_controller.dart';
import 'package:xml/xml.dart';
import '../../../core/models/icon_template_model.dart';
import 'package:ricky_icon_generator/utils/app_image_utils.dart';
import '../../../core/constants.dart';
import '../../../utils/exceptions/cli_exception.dart';
import '../../../utils/general_utils.dart';
import '../base_app_icon_controller.dart';

import 'dart:ui' as ui;
import 'package:image/image.dart' as img;

class AndroidAppIconController extends BaseAppIconController<AndroidIconTemplateModel> {
  AndroidAppIconController({
    required String backgroundColor,
    required Image customSourceImage,
    double? resizePercentage,
  }) : super(backgroundColor: backgroundColor, customSourceImage: customSourceImage, resizePercentage: resizePercentage);

  @experimental
  AndroidAppIconController.custom(
      {required String backgroundColor,
        required Image customSourceImage,
        required double resizePercentage,
        ErrorHandler? errorHandler,
        String? rootPath})
      : super.custom(
      backgroundColor: backgroundColor,
      customSourceImage: customSourceImage,
      resizePercentage: resizePercentage,
      errorHandler: errorHandler,
      rootPath: rootPath);

  @override
  String get platform => kAndroidPlatform;

  @override
  List<AndroidIconTemplateModel> get appIconList =>
      AndroidIconTemplateModel.generateAppIconTemplateModelListByIconTypeList(<AndroidIconTemplateModel>[
        AndroidIconTemplateModel(path: 'mipmap-ldpi', dimens: 0.75),
        AndroidIconTemplateModel(path: 'mipmap-mdpi', dimens: 1.0),
        AndroidIconTemplateModel(path: 'mipmap-hdpi', dimens: 1.5),
        AndroidIconTemplateModel(path: 'mipmap-xhdpi', dimens: 2.0),
        AndroidIconTemplateModel(path: 'mipmap-xxhdpi', dimens: 3.0),
        AndroidIconTemplateModel(path: 'mipmap-xxxhdpi', dimens: 4.0),
      ]);

  @override
  void generateAppIcon() async {
    logger('Generating icons');

    if (customSourceImage.height != 512 && customSourceImage.width != 512) {
      ///Resize source image to xxxhdpi
      customSourceImage = AppImageUtils.resizeImage(
        image: customSourceImage,
        width: 512,
        height: 512,
      );
    }

    //foreground
    // Create alpha layer for xxxhdpi foreground icon (the biggest dimension in android)
    final baseAlphaChannelLayerImage = Image(432, 432);

    //Resized source image by resize percentage
    final resizedBaseForegroundImage = copyResize(
      customSourceImage,
      width: (288 * resizePercentage).toInt(),
      height: (288 * resizePercentage).toInt(),
      interpolation: Interpolation.average,
    );

    // Define paddings for [resizedBaseForegroundImage]
    final paddingX = (432 - resizedBaseForegroundImage.width) ~/ 2;
    final paddingY = (432 - resizedBaseForegroundImage.height) ~/ 2;

    final foregroundImage = drawImage(baseAlphaChannelLayerImage, resizedBaseForegroundImage, dstX: paddingX, dstY: paddingY);
    //change padding depending on resizedBaseForegroundImage size

    //legacy
    final resizedLegacyIcon = await _croppedLegacyImageFromSource(foregroundImage, foregroundImage.width ~/ 2, foregroundImage.height ~/ 2, 288, 288);

    //round
    final resizedRoundIcon = _croppedRoundImageFromSource(foregroundImage);

    for (var template in appIconList) {
      if (template.type == AndroidIconTemplateModelType.icLauncher) {
        AppImageUtils.saveImage(resFolder: getFullPath(kAndroidResFolder), template: template, image: resizedLegacyIcon);
      } else if (template.type == AndroidIconTemplateModelType.icLauncherRound) {
        AppImageUtils.saveImage(resFolder: getFullPath(kAndroidResFolder), template: template, image: resizedRoundIcon);
      } else {
        AppImageUtils.saveImage(resFolder: getFullPath(kAndroidResFolder), template: template, image: foregroundImage);
      }
    }
  }

  Future<Image> _croppedLegacyImageFromSource(Image source, int x, int y, int width, int height) async {
    final tlx = x - 144; //topLeft.x
    final tly = y - 144; //topLeft.y

    final sourceWithSafeAreaBackground = Image.from(source)..fillBackground(ColorUtils.hexToColor(backgroundColor!).value);

    final croppedImage = await _cropToSquircle(sourceWithSafeAreaBackground, tlx, tly, width, height);

    return _croppedImageWithAlpha(croppedImage, 150, 150, 20, 20);
  }

  Future<Image> _cropToSquircle(Image sourceWithSafeAreaBackground, int tlx, int tly, int width, int height) {
    final image = copyCrop(sourceWithSafeAreaBackground, tlx, tly, width, height);

    final tmpImageAsUint8List = Uint8List.fromList(img.encodePng(image, level: 0));

    //init Image codec, take a frame of rendered image, assign it to resized one.
    return ui.instantiateImageCodec(tmpImageAsUint8List).then((codec) {
      return codec.getNextFrame().then((frameInfo) async {
        final recorder = ui.PictureRecorder();
        final rect = ui.Rect.fromPoints(const ui.Offset(0.0, 0.0), ui.Offset(image.width.toDouble(), image.height.toDouble()));
        final canvas = ui.Canvas(recorder, rect);

        canvas.clipRRect(ui.RRect.fromRectAndCorners(rect,
            topLeft: const ui.Radius.circular(16),
            topRight: const ui.Radius.circular(16),
            bottomLeft: const ui.Radius.circular(16),
            bottomRight: const ui.Radius.circular(16)));

        canvas.drawImage(frameInfo.image, const ui.Offset(0, 0), ui.Paint());

        final picture = recorder.endRecording();
        final squircleImage = await picture.toImage(frameInfo.image.width, frameInfo.image.height);

        final pngInBytes = await squircleImage.toByteData(format: ui.ImageByteFormat.png);
        final pngAsUint8List = pngInBytes!.buffer.asUint8List();

        return img.decodeImage(pngAsUint8List)!;
      });
    });
  }

  Image _croppedRoundImageFromSource(Image source) {
    final sourceWithSafeAreaBackground = Image.from(source)..fillBackground(ColorUtils.hexToColor(backgroundColor!).value);

    final croppedImage = copyCropCircle(sourceWithSafeAreaBackground, radius: 144);
    return _croppedImageWithAlpha(croppedImage, 178, 178, 6, 6);
  }

  Image _croppedImageWithAlpha(Image source, int resizedImageWidth, int resizedImageHeight, int paddingX, int paddingY) {
    final baseAlphaChannelLayerImage = Image(192, 192);

    final resizedRoundImage = copyResize(
      source,
      width: resizedImageWidth,
      height: resizedImageHeight,
      interpolation: Interpolation.average,
    );

    final roundPaddingX = paddingX;
    final roundPaddingY = paddingY;

    return drawImage(baseAlphaChannelLayerImage, resizedRoundImage, dstX: roundPaddingX, dstY: roundPaddingY);
  }

  @override
  void executeConfigurationProcess() {
    _generateAdaptiveIconXml();
    _applyIconParametersToManifest();
    _addBackgroundColor();
  }

  void _applyIconParametersToManifest() {
    final androidManifestFile = File(getFullPath(kAndroidManifestFile));
    if (androidManifestFile.existsSync()) {
      final androidManifestXMLDocument = XmlDocument.parse(androidManifestFile.readAsStringSync());
      final applicationElement = androidManifestXMLDocument.getElement('manifest')?..getElement('application');

      if (applicationElement == null) {
        throw CliException('$platform application element was not found. Exit.');
      }

      applicationElement.attributes
        ..removeWhere((element) => element.name.toString() == 'android:icon' || element.name.toString() == 'android:roundIcon')
        ..add(XmlAttribute(XmlName('android:icon'), '@mipmap/ic_launcher'))
        ..add(XmlAttribute(XmlName('android:roundIcon'), '@mipmap/ic_launcher_round'));

      androidManifestFile.writeAsStringSync(androidManifestXMLDocument.toXmlString(pretty: true, indent: '   '));
    } else {
      throw CliException('AndroidManifest.xml was not found. Exit.');
    }
  }

  void _generateAdaptiveIconXml() {
    logger('Creating adaptive icon config');
    final launcherFile = File(getFullPath(kAndroidLauncher));

    if (launcherFile.existsSync()) {
      launcherFile.delete(recursive: true);
    }

    launcherFile.createSync(recursive: true);

    logger('Adding background color');
    final launcherFileBuilder = XmlBuilder();

    launcherFileBuilder
      ..processing('xml', 'version="1.0" encoding="utf-8"')
      ..element('adaptive-icon');
    final launcherFileDocument = launcherFileBuilder.buildDocument();

    final adaptiveIconElement = launcherFileDocument.getElement('adaptive-icon')!
      ..setAttribute('xmlns:android', 'http://schemas.android.com/apk/res/android');
    final List<XmlNode> adaptiveIconElementChildren = adaptiveIconElement.children;

    final backgroundItemBuilder = XmlBuilder();
    backgroundItemBuilder.element('background', nest: () {
      backgroundItemBuilder.attribute('android:drawable', '@color/launcherBackground');
    });

    final foregroundItemBuilder = XmlBuilder();
    foregroundItemBuilder.element('foreground', nest: () {
      foregroundItemBuilder.attribute('android:drawable', '@mipmap/ic_launcher_foreground');
    });

    adaptiveIconElementChildren
      ..add(backgroundItemBuilder.buildFragment())
      ..add(foregroundItemBuilder.buildFragment());

    launcherFile.writeAsStringSync(launcherFileDocument.toXmlString(pretty: true, indent: '   '));

    logger('Creating adaptive round icon config');
    launcherFile.copySync(getFullPath(kAndroidLauncherRound));
  }

  void _addBackgroundColor() {
    final colorsFile = File(getFullPath(kAndroidColorsFile));
    if (colorsFile.existsSync()) {
      logger('colors.xml existing already, add launcher background color');
      final colorsDocument = XmlDocument.parse(colorsFile.readAsStringSync());
      final resources = colorsDocument.getElement('resources')?.name;

      try {
        final launcherBackground =
        resources?.parent?.children.firstWhere((element) => element.attributes.any((attribute) => attribute.value == 'launcherBackground'));

        launcherBackground?.innerText = backgroundColor!;
      } on StateError {
        logger('launcherBackground was not found in colors.xml. Adding lancherBackground color.');

        resources?.parent?.children
            .add(XmlElement(XmlName('color'), [XmlAttribute(XmlName('name'), 'launcherBackground')], [XmlText(backgroundColor!)]));

        colorsFile.writeAsStringSync(colorsDocument.toXmlString(pretty: true, indent: '    '));
      }
    } else {
      logger("colors.xml doesn't exist, create file and add launcher background color");
      colorsFile.createSync(recursive: true);

      final colorsFileBuilder = XmlBuilder()
        ..processing('xml', 'version="1.0" encoding="utf-8"')
        ..element('resources');

      final colorsFileDocument = colorsFileBuilder.buildDocument();

      final colorsFileElement = colorsFileDocument.getElement('resources');

      final List<XmlNode> colorsFileElementChildren = colorsFileElement!.children;

      final backgroundItemBuilder = XmlBuilder();
      backgroundItemBuilder.element('color', nest: () {
        backgroundItemBuilder
          ..attribute('name', 'launcherBackground')
          ..text(backgroundColor!);
      });

      colorsFileElementChildren.add(backgroundItemBuilder.buildFragment());
      colorsFile.writeAsStringSync(colorsFileDocument.toXmlString(pretty: true, indent: '   '));
    }
  }
}

==> ricky_icon_generator-0.0.4.tar.gz: lib/commands/app_icon/android/android_app_icon_controller.dart <==
import 'dart:io';
import 'dart:typed_data';

import 'package:image/image.dart';
import 'package:meta/meta.dart';
import 'package:ricky_icon_generator/core/base_controller.dart';
import 'package:xml/xml.dart';
import '../../../core/models/icon_template_model.dart';
import 'package:ricky_icon_generator/utils/app_image_utils.dart';
import '../../../core/constants.dart';
import '../../../utils/exceptions/cli_exception.dart';
import '../../../utils/general_utils.dart';
import '../base_app_icon_controller.dart';

import 'dart:ui' as ui;
import 'package:image/image.dart' as img;

class AndroidAppIconController extends BaseAppIconController<AndroidIconTemplateModel> {
  AndroidAppIconController({
    required String backgroundColor,
    required Image customSourceImage,
    double? resizePercentage,
  }) : super(backgroundColor: backgroundColor, customSourceImage: customSourceImage, resizePercentage: resizePercentage);

  @experimental
  AndroidAppIconController.custom(
      {required String backgroundColor,
        required Image customSourceImage,
        required double resizePercentage,
        ErrorHandler? errorHandler,
        String? rootPath})
      : super.custom(
      backgroundColor: backgroundColor,
      customSourceImage: customSourceImage,
      resizePercentage: resizePercentage,
      errorHandler: errorHandler,
      rootPath: rootPath);

  @override
  String get platform => kAndroidPlatform;

  @override
  List<AndroidIconTemplateModel> get appIconList =>
      AndroidIconTemplateModel.generateAppIconTemplateModelListByIconTypeList(<AndroidIconTemplateModel>[
        AndroidIconTemplateModel(path: 'mipmap-ldpi', dimens: 0.75),
        AndroidIconTemplateModel(path: 'mipmap-mdpi', dimens: 1.0),
        AndroidIconTemplateModel(path: 'mipmap-hdpi', dimens: 1.5),
        AndroidIconTemplateModel(path: 'mipmap-xhdpi', dimens: 2.0),
        AndroidIconTemplateModel(path: 'mipmap-xxhdpi', dimens: 3.0),
        AndroidIconTemplateModel(path: 'mipmap-xxxhdpi', dimens: 4.0),
      ]);

  @override
  void generateAppIcon() async {
    logger('Generating icons');

    if (customSourceImage.height >= 512 && customSourceImage.width >= 512) {
      ///Resize source image to xxxhdpi
      customSourceImage = AppImageUtils.resizeImage(
        image: customSourceImage,
        width: 512,
        height: 512,
      );
    }

    //foreground
    // Create alpha layer for xxxhdpi foreground icon (the biggest dimension in android)
    final baseAlphaChannelLayerImage = Image(432, 432);

    //Resized source image by resize percentage
    final resizedBaseForegroundImage = copyResize(
      customSourceImage,
      width: (288 * resizePercentage).toInt(),
      height: (288 * resizePercentage).toInt(),
      interpolation: Interpolation.average,
    );

    // Define paddings for [resizedBaseForegroundImage]
    final paddingX = (432 - resizedBaseForegroundImage.width) ~/ 2;
    final paddingY = (432 - resizedBaseForegroundImage.height) ~/ 2;

    final foregroundImage = drawImage(baseAlphaChannelLayerImage, resizedBaseForegroundImage, dstX: paddingX, dstY: paddingY);
    //change padding depending on resizedBaseForegroundImage size

    //legacy
    final resizedLegacyIcon = await _croppedLegacyImageFromSource(foregroundImage, foregroundImage.width ~/ 2, foregroundImage.height ~/ 2, 288, 288);

    //round
    final resizedRoundIcon = _croppedRoundImageFromSource(foregroundImage);

    for (var template in appIconList) {
      if (template.type == AndroidIconTemplateModelType.icLauncher) {
        AppImageUtils.saveImage(resFolder: getFullPath(kAndroidResFolder), template: template, image: resizedLegacyIcon);
      } else if (template.type == AndroidIconTemplateModelType.icLauncherRound) {
        AppImageUtils.saveImage(resFolder: getFullPath(kAndroidResFolder), template: template, image: resizedRoundIcon);
      } else {
        AppImageUtils.saveImage(resFolder: getFullPath(kAndroidResFolder), template: template, image: foregroundImage);
      }
    }
  }

  Future<Image> _croppedLegacyImageFromSource(Image source, int x, int y, int width, int height) async {
    final tlx = x - 144; //topLeft.x
    final tly = y - 144; //topLeft.y

    final sourceWithSafeAreaBackground = Image.from(source)..fillBackground(ColorUtils.hexToColor(backgroundColor!).value);

    final croppedImage = await _cropToSquircle(sourceWithSafeAreaBackground, tlx, tly, width, height);

    return _croppedImageWithAlpha(croppedImage, 150, 150, 20, 20);
  }

  Future<Image> _cropToSquircle(Image sourceWithSafeAreaBackground, int tlx, int tly, int width, int height) {
    final image = copyCrop(sourceWithSafeAreaBackground, tlx, tly, width, height);

    final tmpImageAsUint8List = Uint8List.fromList(img.encodePng(image, level: 0));

    //init Image codec, take a frame of rendered image, assign it to resized one.
    return ui.instantiateImageCodec(tmpImageAsUint8List).then((codec) {
      return codec.getNextFrame().then((frameInfo) async {
        final recorder = ui.PictureRecorder();
        final rect = ui.Rect.fromPoints(const ui.Offset(0.0, 0.0), ui.Offset(image.width.toDouble(), image.height.toDouble()));
        final canvas = ui.Canvas(recorder, rect);

        canvas.clipRRect(ui.RRect.fromRectAndCorners(rect,
            topLeft: const ui.Radius.circular(16),
            topRight: const ui.Radius.circular(16),
            bottomLeft: const ui.Radius.circular(16),
            bottomRight: const ui.Radius.circular(16)));

        canvas.drawImage(frameInfo.image, const ui.Offset(0, 0), ui.Paint());

        final picture = recorder.endRecording();
        final squircleImage = await picture.toImage(frameInfo.image.width, frameInfo.image.height);

        final pngInBytes = await squircleImage.toByteData(format: ui.ImageByteFormat.png);
        final pngAsUint8List = pngInBytes!.buffer.asUint8List();

        return img.decodeImage(pngAsUint8List)!;
      });
    });
  }

  Image _croppedRoundImageFromSource(Image source) {
    final sourceWithSafeAreaBackground = Image.from(source)..fillBackground(ColorUtils.hexToColor(backgroundColor!).value);

    final croppedImage = copyCropCircle(sourceWithSafeAreaBackground, radius: 144);
    return _croppedImageWithAlpha(croppedImage, 178, 178, 6, 6);
  }

  Image _croppedImageWithAlpha(Image source, int resizedImageWidth, int resizedImageHeight, int paddingX, int paddingY) {
    final baseAlphaChannelLayerImage = Image(192, 192);

    final resizedRoundImage = copyResize(
      source,
      width: resizedImageWidth,
      height: resizedImageHeight,
      interpolation: Interpolation.average,
    );

    final roundPaddingX = paddingX;
    final roundPaddingY = paddingY;

    return drawImage(baseAlphaChannelLayerImage, resizedRoundImage, dstX: roundPaddingX, dstY: roundPaddingY);
  }

  @override
  void executeConfigurationProcess() {
    _generateAdaptiveIconXml();
    _applyIconParametersToManifest();
    _addBackgroundColor();
  }

  void _applyIconParametersToManifest() {
    final androidManifestFile = File(getFullPath(kAndroidManifestFile));
    if (androidManifestFile.existsSync()) {
      final androidManifestXMLDocument = XmlDocument.parse(androidManifestFile.readAsStringSync());
      final applicationElement = androidManifestXMLDocument.getElement('manifest')?..getElement('application');

      if (applicationElement == null) {
        throw CliException('$platform application element was not found. Exit.');
      }

      applicationElement.attributes
        ..removeWhere((element) => element.name.toString() == 'android:icon' || element.name.toString() == 'android:roundIcon')
        ..add(XmlAttribute(XmlName('android:icon'), '@mipmap/ic_launcher'))
        ..add(XmlAttribute(XmlName('android:roundIcon'), '@mipmap/ic_launcher_round'));

      androidManifestFile.writeAsStringSync(androidManifestXMLDocument.toXmlString(pretty: true, indent: '   '));
    } else {
      throw CliException('AndroidManifest.xml was not found. Exit.');
    }
  }

  void _generateAdaptiveIconXml() {
    logger('Creating adaptive icon config');
    final launcherFile = File(getFullPath(kAndroidLauncher));

    if (launcherFile.existsSync()) {
      launcherFile.delete(recursive: true);
    }

    launcherFile.createSync(recursive: true);

    logger('Adding background color');
    final launcherFileBuilder = XmlBuilder();

    launcherFileBuilder
      ..processing('xml', 'version="1.0" encoding="utf-8"')
      ..element('adaptive-icon');
    final launcherFileDocument = launcherFileBuilder.buildDocument();

    final adaptiveIconElement = launcherFileDocument.getElement('adaptive-icon')!
      ..setAttribute('xmlns:android', 'http://schemas.android.com/apk/res/android');
    final List<XmlNode> adaptiveIconElementChildren = adaptiveIconElement.children;

    final backgroundItemBuilder = XmlBuilder();
    backgroundItemBuilder.element('background', nest: () {
      backgroundItemBuilder.attribute('android:drawable', '@color/launcherBackground');
    });

    final foregroundItemBuilder = XmlBuilder();
    foregroundItemBuilder.element('foreground', nest: () {
      foregroundItemBuilder.attribute('android:drawable', '@mipmap/ic_launcher_foreground');
    });

    adaptiveIconElementChildren
      ..add(backgroundItemBuilder.buildFragment())
      ..add(foregroundItemBuilder.buildFragment());

    launcherFile.writeAsStringSync(launcherFileDocument.toXmlString(pretty: true, indent: '   '));

    logger('Creating adaptive round icon config');
    launcherFile.copySync(getFullPath(kAndroidLauncherRound));
  }

  void _addBackgroundColor() {
    final colorsFile = File(getFullPath(kAndroidColorsFile));
    if (colorsFile.existsSync()) {
      logger('colors.xml existing already, add launcher background color');
      final colorsDocument = XmlDocument.parse(colorsFile.readAsStringSync());
      final resources = colorsDocument.getElement('resources')?.name;

      try {
        final launcherBackground =
        resources?.parent?.children.firstWhere((element) => element.attributes.any((attribute) => attribute.value == 'launcherBackground'));

        launcherBackground?.innerText = backgroundColor!;
      } on StateError {
        logger('launcherBackground was not found in colors.xml. Adding lancherBackground color.');

        resources?.parent?.children
            .add(XmlElement(XmlName('color'), [XmlAttribute(XmlName('name'), 'launcherBackground')], [XmlText(backgroundColor!)]));

        colorsFile.writeAsStringSync(colorsDocument.toXmlString(pretty: true, indent: '    '));
      }
    } else {
      logger("colors.xml doesn't exist, create file and add launcher background color");
      colorsFile.createSync(recursive: true);

      final colorsFileBuilder = XmlBuilder()
        ..processing('xml', 'version="1.0" encoding="utf-8"')
        ..element('resources');

      final colorsFileDocument = colorsFileBuilder.buildDocument();

      final colorsFileElement = colorsFileDocument.getElement('resources');

      final List<XmlNode> colorsFileElementChildren = colorsFileElement!.children;

      final backgroundItemBuilder = XmlBuilder();
      backgroundItemBuilder.element('color', nest: () {
        backgroundItemBuilder
          ..attribute('name', 'launcherBackground')
          ..text(backgroundColor!);
      });

      colorsFileElementChildren.add(backgroundItemBuilder.buildFragment());
      colorsFile.writeAsStringSync(colorsFileDocument.toXmlString(pretty: true, indent: '   '));
    }
  }
}
